"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.translatePushReplaceArgs = void 0;
var fileUrlToUrl_1 = require("./fileUrlToUrl");
var getLocale_1 = require("./getLocale");
var ntrData_1 = require("./ntrData");
var removeLangPrefix_1 = require("./removeLangPrefix");
var urlToFileUrl_1 = require("./urlToFileUrl");
var translatePushReplaceArgs = function (_a) {
    var router = _a.router, url = _a.url, as = _a.as, locale = _a.locale;
    if (as) {
        return { url: url, as: as, locale: locale };
    }
    var newLocale = (0, getLocale_1.getLocale)(router, locale);
    var locales = router.locales || (0, ntrData_1.getNtrData)().locales;
    var unprefixedUrl = typeof url === 'string' ? (0, removeLangPrefix_1.removeLangPrefix)(url) : url;
    var urlLocale = typeof url === 'string' && unprefixedUrl !== url ? url.split('/')[1] : undefined;
    // propLocale === false if opted-out of automatically handling the locale prefixing
    // Cf. https://nextjs.org/docs/advanced-features/i18n-routing#transition-between-locales
    if (locale === false && urlLocale && locales.includes(urlLocale)) {
        newLocale = urlLocale;
    }
    var parsedUrl;
    var translatedUrl;
    /**
     * Href can be:
     * - an external url
     * - a correct file url
     * - a wrong file url (not matching any page)
     * - a correct translated url
     * - a wrong translated url
     */
    try {
        translatedUrl = (0, fileUrlToUrl_1.fileUrlToUrl)(unprefixedUrl, newLocale);
        // Url is a correct file url
        parsedUrl = unprefixedUrl;
    }
    catch (_b) {
        parsedUrl = (0, urlToFileUrl_1.urlToFileUrl)(unprefixedUrl, urlLocale || newLocale);
        if (parsedUrl) {
            try {
                translatedUrl = (0, fileUrlToUrl_1.fileUrlToUrl)(parsedUrl, newLocale);
                // Href is a correct translated url
            }
            catch (_c) {
                // Href is a wrong file url or an external url
            }
        }
        else {
            // Href is a wrong url or an external url
        }
    }
    return { url: parsedUrl || url, as: translatedUrl, locale: newLocale };
};
exports.translatePushReplaceArgs = translatePushReplaceArgs;
//# sourceMappingURL=translatePushReplaceArgs.js.map